/* ------------------------------------------------------------------------ *
 * SAS Institute, Inc.                                                      *
 *                                                                          *
 * Copyright (c) 2006-2013 SAS Institute, Inc.  All rights reserved.             *
 *                                                                          *
 * Contains functions that provide support for interacting with a report    *
 * open for viewing and performing tasks associated with a rendered report. *
 * ------------------------------------------------------------------------ */
wrsProvide("citation_reportControl");
wrsRequire("citation",  SCRIPT_DIR, "citation reportControl");
 
function ReportControl() 
{
    this._windowUpdateAllowed = true;
    //should be overridden by instantiating page
    this._defaultMsgConfirmUnsavedChanges = 'The report has unsaved changes. Do you wish to continue anyway?';
    this._msgUnsavedChangesOnReturnToPreviousReport = null;
    this._previousReportLink = null;
}

ReportControl.ACTION_SOURCE_ADAPTER_ID_KEY = "DRAW_ELEMENT_ID_PARM";

ReportControl.prototype.setDefaultUnsavedChangesMsg = function(msg) { this.msgConfirmUnsavedChanges = msg; }
ReportControl.prototype.getDefaultUnsavedChangesMsg = function() { return this.msgConfirmUnsavedChanges; }

ReportControl.prototype.updateWindow = function(windowLocation)
{
    if (!this._windowUpdateAllowed) {
        // alert("reportControl", "blocked window update: " + windowLocation);
        return;
    }
    this._windowUpdateAllowed = false;
    window.location = windowLocation;
}
ReportControl.prototype.refreshWindow = function(windowLocation)
{
    if (!this._windowUpdateAllowed) {
        // alert("reportControl", "blocked window refresh: " + windowLocation);
        return;
    }
    onLeavePage();
    this._windowUpdateAllowed = false;
    window.location = windowLocation;
}

//static utility
ReportControl.attachUrlParms = function(urlStr, parms)
{
    if (parms == null || parms.length < 1){
        return urlStr;
    }    
    var rval = urlStr;
    if (rval != null && rval.indexOf('javascript')) {
        var lastQPos = rval.lastIndexOf('?');
        if (lastQPos == -1) {
            rval += "?";
        }        
        if (rval.lastIndexOf('?') != rval.length-1) {   // parms already present after '?'
            if (rval.lastIndexOf('&') != rval.length-1){
               if (parms.charAt(0) != '&') {
                  rval += "&";
               }
            }
        }
        rval += parms;
    }
    return rval; 
}

ReportControl.joinUrlParms = function(parms1, parms2)
{
   if (parms1 == null) {
       if (parms2 == null)
           return null;
       return "" + parms2
   }
      
   if (parms2 == null)
       return "" + parms1;
       
    var rval = parms1;
    var lastQPos = rval.lastIndexOf('?');
    if (rval.lastIndexOf('&') != rval.length-1){
       if (parms2.charAt(0) != '&') {
          rval += "&";
       }
    }
    rval += parms2;
    return rval;     
}

// XXXXXXXXXXXXXXXXXXXXXXXXXX LOGIC: prompting for unsaved changes (start) XXXXXXXXXXXXXXXXXXXXXXXXXXXXX

// S0844274
// Calls back to actionIfYes function if REPORT has unsaved changes, or actionIfNo if not
ReportControl.prototype.callbackForkUnsavedChanges = function(actionIfYes, actionIfNo)
{
   showDebugMsgs("g_reportControl", "in unsavedChangesFork()");
   
   var link = "checkUnsavedReportChangesIgnoreCapabilities.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: unsavedChangesFork", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from unsavedChangesFork");
                if (handleSessionTimeout(data, evt)) return;
                if (data != null && data != "undefined")
                {
                    if (data == "reportAltered" && actionIfYes != null && actionIfYes != "undefined")
                    {
                        actionIfYes();
                    }
                    else if (actionIfNo != null && actionIfNo != "undefined")
                    {
                        actionIfNo();                   
                    }
                 }
             }
     });          
    return;
}

ReportControl.prototype.checkForUnsavedChanges = function(actionToPerform, confirmMsg, additionalParm)
{
   showDebugMsgs("g_reportControl", "about to check for unsaved Changes"+actionToPerform);
   var activeMsgConfirmUnsavedChanges = confirmMsg == null ? null : confirmMsg;
   
   var link = "checkUnsavedReportChanges.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: checkForUnsavedChanges", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from checkForUnsavedChanges");
                if (handleSessionTimeout(data, evt)) return;
                if (additionalParm)
                   ReportControl.prototype.handleUnsavedChangesCheck(data, evt, actionToPerform, activeMsgConfirmUnsavedChanges, additionalParm);
                else
                   ReportControl.prototype.handleUnsavedChangesCheck(data, evt, actionToPerform, activeMsgConfirmUnsavedChanges);
             }
     });          
    return;
}

// Checks for unsaved changes without regard for user's capabilities
ReportControl.prototype.checkForUnsavedChangesIgnoreCapabilities = function(actionToPerform, confirmMsg, additionalParm)
{
   showDebugMsgs("g_reportControl", "about to check for unsaved Changes"+actionToPerform);
   var activeMsgConfirmUnsavedChanges = confirmMsg == null ? null : confirmMsg;
   
   var link = "checkUnsavedReportChangesIgnoreCapabilities.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: checkForUnsavedChanges", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from checkForUnsavedChanges");
                if (handleSessionTimeout(data, evt)) return;
                if (additionalParm)
                   ReportControl.prototype.handleUnsavedChangesCheck(data, evt, actionToPerform, activeMsgConfirmUnsavedChanges, additionalParm);
                else
                   ReportControl.prototype.handleUnsavedChangesCheck(data, evt, actionToPerform, activeMsgConfirmUnsavedChanges);
             }
     });          
    return;
}

ReportControl.prototype.handleUnsavedChangesCheck = function(data, evt, actionToPerform, activeMsgConfirmUnsavedChanges, additionalParm)
{
    var goAhead = true;
    if (data != null && data != "undefined")
    {
       if (data == "reportAltered")
       {
           var msg = activeMsgConfirmUnsavedChanges;
           if (msg == null)
           {
               msg = this.getDefaultUnsavedChangesMsg();
           }
          goAhead = confirm(msg);
       }
    }
    if (goAhead && actionToPerform != null)
    {
        if (additionalParm)
           actionToPerform(additionalParm);
        else
           actionToPerform();
    }
}

ReportControl.prototype._openPreviousReportCore = function(previousReportLink)
{
    if (previousReportLink != null)
    {
        window.location.href = previousReportLink;
    }
} 
ReportControl.prototype.openPreviousReport = function()
{
    this.checkForUnsavedChanges(this._openPreviousReportCore, this._msgUnsavedChangesOnReturnToPreviousReport, this._previousReportLink);
}

/*  ------------------------------------------------------------------------------------
    All of the "*Confirmed" functions are callbacks from confirmReportChanges.jsp
    That file contains the callback functionality when the user needs to be prompted
    as to whether to save changes to a report before open another, creating a new one, etc.
    The "dropCurrentReport" parm indicates that an additional Struts action to clear
    the report rendering cache for the previous report needs to happen before redirecting
    to the ultimate action.   This saves what would have been yet another round trip to the
    server just to clear out the old report.
    ------------------------------------------------------------------------------------- */
//function cwNewReportConfirmed(dropCurrentReport)
ReportControl.prototype.cwLeaveReportConfirmed = function(destinationURL)
{
    this.updateWindow(destinationURL);
}

// XXXXXXXXXXXXXXXXXXXXXXXXXX LOGIC: prompting for unsaved changes (end) XXXXXXXXXXXXXXXXXXXXXXXXXXXXX


//  S0844274:
//  Push a pending item (e.g., the equivalent of a File Email command) 
//  that will be played back after current page is replaced with an entirely new page
//  (e.g., as a result of user saving successfully after choosing 
//  Yes in the SaveBeforeEmailDialog)
ReportControl.prototype.pushPendingEmailAction = function()
{
   showDebugMsgs("g_reportControl", "about to request push pending email action");
   
   var link = "pushPendingEmail.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: pushPendingEmailAction", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from pushPendingEmailAction");
                if (handleSessionTimeout(data, evt)) return;
                // Do nothing else because entire page will be refreshed.
             }
     });          
    return;
}

//  S0844274:
//  If there is a pending item for this page (e.g., the equivalent of a 
//  File Email command that was postponed while the page was refreshed after
//  a successful Save-As when user chose Yes from the SaveBeforeEmailDialog),
//  then pop that item and call the appropriate function to kick off that action.
//  If there is no pending item (as is normally the case), then do nothing.
ReportControl.prototype.popPendingItem = function()
{
   showDebugMsgs("g_reportControl", "about to request pop pending item action");
   
   var link = "popPendingItem.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: pushPendingEmailAction", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from pushPendingEmailAction");
                if (handleSessionTimeout(data, evt)) return;
                ReportControl.prototype.handlePopPendingItem(data, evt);
             }
     });          
    return;
}

//  S0844274:
ReportControl.prototype.handlePopPendingItem = function(data, evt)
{
    if (data != null && data != "undefined")
    {
        showDebugMsgs("g_reportControl", "in method handlePopPendingItem(), have data");            
        //alert("In citation_reportControl handlePopPendingItem() data=" + data);
        if ("pendingEmail" == data)
        {
            if (cwMenuBarEmailConfirmed)
            {
                cwMenuBarEmailConfirmed();  // Kick off pending File > Email action
            }               
        }
    }
}

// S0861007
ReportControl.prototype.savePrivateCopyEmail = function()
{
   showDebugMsgs("g_reportControl", "about to request save private copy email");
   
   var link = "privateCopyEmail.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: savePrivateCopyEmailAction", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from savePrivateCopyEmailAction");
                if (handleSessionTimeout(data, evt)) return;
                ReportControl.prototype.handleSavePrivateCopyEmail(data, evt);
             }
     });          
    return;
}

ReportControl.prototype.handleSavePrivateCopyEmail = function(data, evt)
{
    if (data != null && data != "undefined")
    {
        //alert("In citation_reportControl handleSavePrivateCopyEmail(), data=" + data);
        if (data && data.length > 4)
        {
            cwPrivateCopySaved(data);
        }
    }
}


// S0821111:
// Calls back to actionIfYes function if save in place is allowed, or to actionIfNo if not
// (or neither if there's a problem).
ReportControl.prototype.callbackForkIsSaveInPlaceAllowed = function(actionIfYes, actionIfNo)
{
   showDebugMsgs("g_reportControl", "in callbackForkIsSaveInPlaceAllowed()");
   
   var link = "isSaveInPlaceAllowed.do?session=required";

   dojo.io.bind( {
     method: "POST",
     url: link,
     preventCache: true,
     mimetype: "text/plain",
     error: function (type, data, evt) {
                 showDebugMsgs(
                    "ERROR: callbackForkIsSaveInPlaceAllowed", 
                    "ERROR Returned from call." + data.message + evt);
            },
     load: function(type, data, evt) {
                showDebugMsgs("Back from isSaveInPlaceAllowed");
                if (handleSessionTimeout(data, evt)) return;
                if (data != null && data != "undefined")
                {
                    if (data == "saveInPlaceAllowedTrue" && actionIfYes != null && actionIfYes != "undefined")
                    {
                        actionIfYes();
                    }
                    else if (actionIfNo != null && actionIfNo != "undefined")
                    {
                        actionIfNo();                   
                    }
                 }
             }
     });          
    return;
}

ReportControl.prototype.toggleAppliedFilterDisplay = function(toggleOn)
{
    var toggleURL = toggleOn 
        ? "rvToggleAppliedFilterDisplay.do?APPLIED_FILTERS=true"
        : "rvToggleAppliedFilterDisplay.do";
    this.updateWindow(toggleURL);
}


// TODO insert methods that will be common to both view and build modes 
// Methods expected to fall into this category...
//
// 1) Open Edit Filters (callback functions need to be registered
//    that redraw parent window upon close of dialog)
// 2) Open Select Data (callback functions need to be registered
//    that redraw parent window upon close of dialog)
// 3) Section navigation should also be common between build and view.
//    See gotoReportSection() method. This should really be made a method
//    on the ReportControl type at some point. It doesn't yet b/c gotoReportSection()
//    pre-dates the existence of the ReportControl type.   

var g_reportControl = new ReportControl();

//Function dedicated for page navigation within the current open report.
function showPageNumber(pageNum) 
{
    _showPageNumber('tocOpen', pageNum);
}

function _showPageNumber(openID, pageNum) 
{
    onLeavePage(); // S0335487 start progress indicator after TOC link is selected

    //TODO: FIND where form with name="displayPageForm" gets rendered and
    // a) remove V1 hidden fields
    // b) add hidden fields for items we currently add to query string
    //
    // NOTE: I think TableOfContentsBean is the culprit, but there is other activity
    // going on there that I don't want to interfere with 
    var link = "gotoReportPage.do?"+"pageNumber="+pageNum+"&"+openID+"="+g_openTOC;
    location.href = link;
}

/*
 * Issues an HTTP request to render the section at the specified index
 * in the currently open report. 
 * 
 * @param sectionIndexAsString index of section to go to; the index should be 
 *        specified as a string. 
 */
function gotoReportSection(sectionIndexAsString)
{
    var link = "gotoReportSection.do?sectionNumber=" + sectionIndexAsString;
    // this assures that we don't do it multiple times without actually refreshing
    // the page.    
    g_reportControl.updateWindow(link);  

    showDebugMsgs("g_reportControl", "in gotoReportSection() calling onLeavePage()");    
    onLeavePage(); // S0360803 added call to start wait indicator on first page 
                   // after a section switch
}

function openFilterRankCrosstabDialog_rowColumn(rootDlgUrl, existingMeasureFilters)
{
    cwDialogManager.openDialog(
        'openViewModeFilterRankDialog.do?CMDID=510&APNAME=AP3&dialogCommand=openDialog', 
        cwDialogManager.RANK_FILTER, 
        cwDialogManager.RANK_FILTER, 
        true, 
        true, 
        true, 
        null,
        cwRedrawReport);
}
